package ch.aloba.mpeeditor;

import ch.aloba.mpeeditor.configuration.ExampleConfiguration;
import ch.aloba.mpefw.configuration.ConfigurationFactory;

import java.net.URL;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class ExampleEditorPlugin extends AbstractUIPlugin
{

    // The shared instance.
    private static ExampleEditorPlugin plugin;

    /**
     * The constructor.
     */
    public ExampleEditorPlugin()
    {
        plugin = this;
        ConfigurationFactory.setConfiguration(new ExampleConfiguration());
    }

    /**
     * This method is called upon plug-in activation
     */
    public void start(BundleContext context) throws Exception
    {
        super.start(context);
    }

    /**
     * This method is called when the plug-in is stopped
     */
    public void stop(BundleContext context) throws Exception
    {
        super.stop(context);
        plugin = null;
    }

    /**
     * logs an exception.
     * @param e Exception
     */
    public void logException(Exception e)
    {
        getLog().log(new Status(Status.ERROR, getPluginId(), 0, "Exception", e));
    }

    public String getPluginId()
    {

        return "ch.aloba.ExampleEditorPlugin";
    }

    /**
     * Returns the shared instance.
     */
    public static ExampleEditorPlugin getDefault()
    {
        return plugin;
    }

    /**
     * Returns an image descriptor for the image file at the given plug-in relative path.
     * @param path the path
     * @return the image descriptor
     */
    public static ImageDescriptor getImageDescriptor(String path)
    {
        return AbstractUIPlugin.imageDescriptorFromPlugin("ExampleEditorPlugin", path);
    }

    public URL getInstallURL()
    {
        return getDefault().getBundle().getEntry("/"); //$NON-NLS-1$
    }

    public static Shell getActiveWorkbenchShell()
    {
        IWorkbenchWindow window = getActiveWorkbenchWindow();
        if (window != null)
        {
            return window.getShell();
        }
        return null;
    }

    public static IWorkbenchWindow getActiveWorkbenchWindow()
    {
        return getDefault().getWorkbench().getActiveWorkbenchWindow();
    }

    /**
     * Liefert ImageDescriptoren fr das angegebene Bild
     *
     * @param name -
     *          Der Name der Bilddatei ohne Dateinamenerweiterung
     * @return ImageDescriptor-Tupel fr disabled-Icons und enabled-Icons
     */
    public static ImageDescriptor[] getActionImageDescriptors(
            String name) {
      return new ImageDescriptor[] { create("full/dlcl16/", name),
              create("full/elcl16/", name) };
    }

    /**
     * Erzeugt einen ImageDescriptor fr das angegebene Bild
     *
     * @param prefix -
     *          Ordner, in dem sich die Bilder befinden, relativ zu icons
     * @param name -
     *          Name des Bildes
     * @return - ImageDescriptor des Bildes
     */
    private static ImageDescriptor create(String prefix,
            String name) {
      return getImageDescriptor("icons/" + prefix
              + name + ".gif");
    }
}
