/*
 * aloba ag , Copyright (c) 2006
 * created 30.06.2006 by Hansjoerg
 */
package ch.aloba.mpeeditor.editor.pages;

import ch.aloba.model.imodel.IPerson;
import ch.aloba.mpefw.configuration.ConfigurationFactory;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.pde.core.IBaseModel;
import org.eclipse.pde.core.IModelChangeProvider;
import org.eclipse.pde.core.IModelChangedEvent;
import org.eclipse.pde.internal.ui.editor.PDEDetails;
import org.eclipse.pde.internal.ui.editor.PDEFormPage;
import org.eclipse.pde.internal.ui.editor.PDESection;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.IFormPart;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

/**
 * TODO add class description ...
 *
 * @author Hansjoerg
 * @version $Revision: 1.1 $
 */
public class PersonDetailsPage extends PDEDetails
{
    private Label idLabel = null;
    private Text  nameText = null;
    private IPerson input = null;
    private String nameStr = "";

    /**
     *
     */
    public PersonDetailsPage()
    {
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.IContextPart#getContextId()
     */
    public String getContextId()
    {
        return ConfigurationFactory.getConfiguration().getContextId();
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.IContextPart#fireSaveNeeded()
     */
    public void fireSaveNeeded()
    {
        markDirty();
        getPage().getPDEEditor().fireSaveNeeded(getContextId(), false);
    }

    /**
     *
     * @see org.eclipse.pde.internal.ui.editor.IContextPart#getPage()
     */
    public PDEFormPage getPage()
    {
        return (PDEFormPage)getManagedForm().getContainer();
    }

    /**
     *
     * @see org.eclipse.pde.internal.ui.editor.IContextPart#isEditable()
     */
    public boolean isEditable()
    {
        return getPage().getPDEEditor().getAggregateModel().isEditable();
    }


    /**
     * @see org.eclipse.ui.forms.IDetailsPage#createContents(org.eclipse.swt.widgets.Composite)
     */
    public void createContents(Composite parent)
    {
        TableWrapLayout layout = new TableWrapLayout();
        layout.topMargin = 0;
        layout.leftMargin = 5;
        layout.rightMargin = 0;
        layout.bottomMargin = 0;
        parent.setLayout(layout);
        FormToolkit toolkit = getManagedForm().getToolkit();
        Section section = toolkit.createSection(
                        parent, Section.DESCRIPTION | ExpandableComposite.TITLE_BAR);
        section.clientVerticalSpacing = PDESection.CLIENT_VSPACING;
        section.marginHeight = 5;
        section.marginWidth = 5;
        section.setText("Person"); //$NON-NLS-1$
        section.setDescription("Details of a Person"); //$NON-NLS-1$
        TableWrapData td = new TableWrapData(TableWrapData.FILL, TableWrapData.TOP);
        td.grabHorizontal = true;
        section.setLayoutData(td);

        Composite client = toolkit.createComposite(section);
        GridLayout glayout = new GridLayout(2, false);

        GridData gd = new GridData(GridData.FILL_HORIZONTAL);
        client.setLayout(glayout);
        toolkit.createLabel(client, "Id");
        idLabel = toolkit.createLabel(client, "");
        idLabel.setLayoutData(gd);
        toolkit.createLabel(client, "Name");
        nameText = toolkit.createText(client, "");
        nameText.setLayoutData(gd);
        addListeners();

        toolkit.paintBordersFor(client);
        section.setClient(client);
        IBaseModel model = getPage().getModel();
        if (model instanceof IModelChangeProvider)
            ((IModelChangeProvider) model).addModelChangedListener(this);
        markDetailsPart(section);
    }

    /**
     * @see org.eclipse.ui.forms.IPartSelectionListener#selectionChanged(org.eclipse.ui.forms.IFormPart, org.eclipse.jface.viewers.ISelection)
     */
    public void selectionChanged(IFormPart part, ISelection selection)
    {
        IStructuredSelection ssel = (IStructuredSelection) selection;
        if (ssel.size() == 1)
        {
            input = (IPerson) ssel.getFirstElement();
            nameStr = input.getName();
        }
        else
        {
            input = null;
            nameStr = "";
        }
        update();
    }

    /**
     * @see org.eclipse.pde.core.IModelChangedListener#modelChanged(org.eclipse.pde.core.IModelChangedEvent)
     */
    public void modelChanged(IModelChangedEvent event)
    {
        // un-optimized -> if there is a change, simple refresh the complete table
        markStale();
    }

    private void update()
    {
        idLabel.setText(input != null ? input.getId() : "");
        nameText.setText(nameStr);
    }

    public void refresh()
    {
        update();
        super.refresh();
    }

    /* The following code handles changes in the name TextField. Since there is just one
     * TextField in the details section, we can handle this  directly in the detail page itself.
     * If there should be more, it would be advisable to encapsulate this logic in its own
     * class.
     */
    private boolean dirty = false;

    private void addListeners()
    {
        nameText.addKeyListener(new KeyAdapter()
        {
            public void keyReleased(KeyEvent e)
            {
                keyReleaseOccured(e);
            }
        });
        nameText.addModifyListener(new ModifyListener()
        {
            public void modifyText(ModifyEvent e)
            {
                editOccured(e);
            }
        });
        nameText.addFocusListener(new FocusAdapter()
        {
            public void focusLost(FocusEvent e)
            {
                if (dirty) commit();
            }
        });
    }

    /**
     * If dirty, commits the text in the widget to the value and notifies the listener. This call
     * clears the 'dirty' flag.
     */
    public void commit()
    {
        if (dirty)
        {
            nameStr = nameText.getText();
            try
            {
                input.setName(nameStr);
            }
            catch (CoreException e)
            {
                ConfigurationFactory.getConfiguration().logException(e);
            }
        }
        dirty = false;
    }

    public void cancelEdit()
    {
        dirty = false;
    }

    private void editOccured(ModifyEvent e)
    {
        dirty = true;
    }

    private void keyReleaseOccured(KeyEvent e)
    {
        if (e.character == '\r')
        {
            // commit value
            if (dirty) commit();
        }
        else if (e.character == '\u001b')
        { // Escape character
            nameText.setText(nameStr);
            dirty = false;
        }
    }
}


