/*
 * aloba ag , Copyright (c) 2006
 * created 31.03.2006 by Hansjoerg
 */
package ch.aloba.mpefw.configuration;

import ch.aloba.mpefw.editor.AbstractMpeFwEditor;
import ch.aloba.mpefw.editor.AbstractMpeFwUndoManager;
import ch.aloba.mpefw.model.base.AbstractModelMpeFw;

import org.eclipse.core.resources.IFile;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.pde.internal.ui.editor.ISortableContentOutlinePage;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.plugin.AbstractUIPlugin;

/**
 *  Configuration interface.
 *
 * @author Hansjoerg
 * @version $Revision: 1.1 $
 */
public interface IMpeFwConfiguration
{
    /**
     * Logs an Exception.
     * @param e Exception to log
     */
    public void logException(Exception e);

    /**
     * gets the plugin id.
     * @return String Id of the plugin
     */
    public String getPluginId();

    /**
     * returns a resource string.
     * @param key key of the string to be returned
     * @return String resource string
     */
    public String getResourceString(String key);

    /**
     * returns the context id. This is the id of the context, that is stored in the
     * InputContextManager.
     * @return String the context id.
     */
    public String getContextId();

    /**
     * Instantiates the model by providing the file.
     * @param file XML File.
     * @return AbstractModelMpeFw instantiated model.
     */
    public AbstractModelMpeFw instantiateModel(IFile file);

    /**
     * gets the model.
     * @return AbstractModelMpeFw model.
     */
    public AbstractModelMpeFw getModel();

    /**
     * returns the undomanager.
     * @param editor editor.
     * @return AbstractMpeFwUndoManager undomanager of the editor.
     */
    public AbstractMpeFwUndoManager getUndoManager(AbstractMpeFwEditor editor);

    /**
     * Outlinepage.
     * @param editor editor
     * @return ISortableContentOutlinePage outlinepage
     */
    public ISortableContentOutlinePage getContentOutline(AbstractMpeFwEditor editor);

    /**
     * returns the plugin.
     * @return AbstractUIPlugin the plugin
     */
    public AbstractUIPlugin getPlugin();

    /**
     * gets the active workbenchshell.
     * @return Shell active workbench shell
     */
    public Shell getActiveWorkbenchShell();

    /**
     * Image descriptors.
     * @param imageName name of the image
     * @return ImageDescriptor[] image descriptors for image
     */
    public ImageDescriptor[] getActionImageDescriptors(String imageName);

}


