/*
 * aloba ag , Copyright (c) 2006
 * created 07.04.2006 by Hansjoerg
 */
package ch.aloba.mpefw.editor;

import ch.aloba.mpefw.configuration.ConfigurationFactory;
import ch.aloba.mpefw.editor.context.MpeFwInputContext;
import ch.aloba.mpefw.editor.context.MpeFwInputContextManager;
import ch.aloba.mpefw.model.base.IModelObject;

import java.io.File;
import java.util.Locale;

import org.eclipse.core.resources.IFile;
import org.eclipse.pde.internal.ui.editor.ISortableContentOutlinePage;
import org.eclipse.pde.internal.ui.editor.MultiSourceEditor;
import org.eclipse.pde.internal.ui.editor.PDEFormEditor;
import org.eclipse.pde.internal.ui.editor.PDEFormEditorContributor;
import org.eclipse.pde.internal.ui.editor.PDESourcePage;
import org.eclipse.pde.internal.ui.editor.SystemFileEditorInput;
import org.eclipse.pde.internal.ui.editor.context.InputContext;
import org.eclipse.pde.internal.ui.editor.context.InputContextManager;
import org.eclipse.pde.internal.ui.editor.site.SiteInputContext;
import org.eclipse.pde.internal.ui.editor.site.SiteSourcePage;
import org.eclipse.ui.IEditorActionBarContributor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IStorageEditorInput;
import org.eclipse.ui.forms.editor.IFormPage;
import org.eclipse.ui.part.FileEditorInput;

/**
 * Baseclass for Xml-Multipage editors
 *
 * @author Hansjoerg
 * @version $Revision: 1.1 $
 */
public abstract class AbstractMpeFwEditor extends MultiSourceEditor
{

    /**
     * @see org.eclipse.pde.internal.ui.editor.PDEFormEditor#createResourceContexts(org.eclipse.pde.internal.ui.editor.context.InputContextManager,
     *      org.eclipse.ui.IFileEditorInput)
     */
    protected void createResourceContexts(InputContextManager manager, IFileEditorInput input)
    {
        IFile file = input.getFile();
        IFile siteFile = null;
        String name = file.getName().toLowerCase(Locale.ENGLISH);
        if (checkFileName(name))
        {
            siteFile = file;
            if (siteFile.exists())
            {
                IEditorInput in = new FileEditorInput(siteFile);
                manager.putContext(in, new MpeFwInputContext(this, in, file == siteFile));
            }
            manager.monitorFile(siteFile);
        }
    }

    /**
     * @see org.eclipse.pde.internal.ui.neweditor.MultiSourceEditor#createXMLSourcePage(org.eclipse.pde.internal.ui.neweditor.PDEFormEditor,
     *      java.lang.String, java.lang.String)
     */
    protected PDESourcePage createSourcePage(PDEFormEditor editor, String title, String name,
                    String contextId)
    {
        return new MpeFwSourcePage((AbstractMpeFwEditor) editor, title, name);
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.PDEFormEditor#createInputContextManager()
     */
    protected InputContextManager createInputContextManager()
    {
        MpeFwInputContextManager contextManager = new MpeFwInputContextManager(this);
        contextManager.setUndoManager(ConfigurationFactory.getConfiguration().getUndoManager(this));
        return contextManager;
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.context.IInputContextListener#contextAdded(org.eclipse.pde.internal.ui.editor.context.InputContext)
     */
    public void contextAdded(InputContext context)
    {
        addSourcePage(context.getId());
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.context.IInputContextListener#contextRemoved(org.eclipse.pde.internal.ui.editor.context.InputContext)
     */
    public void contextRemoved(InputContext context)
    {
        if (context.isPrimary())
        {
            close(true);
            return;
        }

        IFormPage page = findPage(context.getId());
        if (page != null)
        {
            removePage(context.getId());
        }
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.context.IInputContextListener#monitoredFileAdded(org.eclipse.core.resources.IFile)
     */
    public void monitoredFileAdded(IFile file)
    {
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.context.IInputContextListener#monitoredFileRemoved(org.eclipse.core.resources.IFile)
     */
    public boolean monitoredFileRemoved(IFile file)
    {
        return true;
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.PDEFormEditor#getInputContext(java.lang.Object)
     */
    protected InputContext getInputContext(Object object)
    {
        InputContext context = null;
        if (object instanceof IModelObject)
        {
            context = inputContextManager.findContext(ConfigurationFactory.getConfiguration()
                            .getContextId());
        }
        return context;
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.PDEFormEditor#createSystemFileContexts(org.eclipse.pde.internal.ui.editor.context.InputContextManager,
     *      org.eclipse.pde.internal.ui.editor.SystemFileEditorInput)
     */
    protected void createSystemFileContexts(InputContextManager manager, SystemFileEditorInput input)
    {
        File file = (File) input.getAdapter(File.class);
        File siteFile = null;

        String name = file.getName().toLowerCase(Locale.ENGLISH);
        if (checkFileName(name))
        {
            siteFile = file;
            if (siteFile.exists())
            {
                IEditorInput in = new SystemFileEditorInput(siteFile);
                manager.putContext(in, new SiteInputContext(this, in, file == siteFile));
            }
        }
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.PDEFormEditor#createStorageContexts(org.eclipse.pde.internal.ui.editor.context.InputContextManager,
     *      org.eclipse.ui.IStorageEditorInput)
     */
    protected void createStorageContexts(InputContextManager manager, IStorageEditorInput input)
    {
        String name = input.getName().toLowerCase(Locale.ENGLISH);
        if (checkFileName(name))
        {
            manager.putContext(input, new SiteInputContext(this, input, true));
        }
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.PDEFormEditor#createContentOutline()
     */
    protected ISortableContentOutlinePage createContentOutline()
    {
        return ConfigurationFactory.getConfiguration().getContentOutline(this);
    }

    /**
     *
     * @param filename name of the file
     * @return true, if the filename matches the pattern.
     */
    protected abstract boolean checkFileName(String filename);
}
