/*
 * aloba ag , Copyright (c) 2006
 * created 07.04.2006 by Hansjoerg
 */
package ch.aloba.mpefw.editor;

import ch.aloba.mpefw.configuration.ConfigurationFactory;
import ch.aloba.mpefw.model.base.AbstractModelObject;
import ch.aloba.mpefw.model.base.IModelObject;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.pde.core.IModelChangeProvider;
import org.eclipse.pde.core.IModelChangedEvent;
import org.eclipse.pde.internal.ui.PDEPlugin;
import org.eclipse.pde.internal.ui.editor.IModelUndoManager;
import org.eclipse.pde.internal.ui.editor.ModelUndoManager;

/**
 * Base class for undo manager
 *
 * @author Hansjoerg
 * @version $Revision: 1.1 $
 */
public abstract class AbstractMpeFwUndoManager extends ModelUndoManager
{
    /**
     * Constructor.
     * @param editor editor
     */
    public AbstractMpeFwUndoManager(AbstractMpeFwEditor editor)
    {
        super(editor);
        setUndoLevelLimit(30);
    }

    /**
     * @see IModelUndoManager#execute(ModelUndoOperation)
     */
    protected void execute(IModelChangedEvent event, boolean undo)
    {
        IModelChangeProvider model = event.getChangeProvider();
        Object[] elements = event.getChangedObjects();
        int type = event.getChangeType();
        String propertyName = event.getChangedProperty();

        switch (type)
        {
            case IModelChangedEvent.INSERT:
                if (undo) executeRemove(model, elements);
                else
                    executeAdd(model, elements);
                break;
            case IModelChangedEvent.REMOVE:
                if (undo) executeAdd(model, elements);
                else
                    executeRemove(model, elements);
                break;
            case IModelChangedEvent.CHANGE:
                if (undo) executeChange(elements[0], propertyName, event.getNewValue(), event
                                .getOldValue());
                else
                    executeChange(elements[0], propertyName, event.getOldValue(), event
                                    .getNewValue());
        }
    }

    /**
     * Executes removing of elements.
     * @param model model
     * @param elements elements
     */
    protected abstract void executeRemove(IModelChangeProvider model, Object[] elements);

    /**
     * Executes adding of elements
     * @param model model
     * @param elements elements
     */
    protected abstract void executeAdd(IModelChangeProvider model, Object[] elements);


    private void executeChange(Object element, String propertyName, Object oldValue, Object newValue)
    {

        if (element instanceof AbstractModelObject)
        {
            AbstractModelObject sobj = (AbstractModelObject) element;
            try
            {
                sobj.restoreProperty(propertyName, oldValue, newValue);
            }
            catch (CoreException e)
            {
                ConfigurationFactory.getConfiguration().logException(e);
            }
        }
    }
}


