/*
 * aloba ag , Copyright (c) 2006
 * created 07.04.2006 by Hansjoerg
 */
package ch.aloba.mpefw.editor.context;

import ch.aloba.mpefw.configuration.ConfigurationFactory;
import ch.aloba.mpefw.editor.AbstractMpeFwEditor;
import ch.aloba.mpefw.model.base.AbstractModelMpeFw;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.pde.core.IBaseModel;
import org.eclipse.pde.core.IEditable;
import org.eclipse.pde.core.IModelChangedEvent;
import org.eclipse.pde.internal.ui.editor.context.XMLInputContext;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IStorageEditorInput;

/**
 * Input Context.
 *
 * @author Hansjoerg
 * @version $Revision: 1.1 $
 */
public class MpeFwInputContext extends XMLInputContext
{

    /**
     * Constructor.
     * @param editor editor
     * @param input IEditorInput instance
     * @param primary primary flag
     */
    public MpeFwInputContext(AbstractMpeFwEditor editor, IEditorInput input, boolean primary)
    {
        super(editor, input, primary);
        create();
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.context.InputContext#createModel(org.eclipse.ui.IEditorInput)
     */
    protected IBaseModel createModel(IEditorInput input)
    {
        IBaseModel model = null;
        if (input instanceof IStorageEditorInput)
        {
            if (input instanceof IFileEditorInput)
            {
                IFile file = ((IFileEditorInput) input).getFile();
                model = createWorkspaceModel(file);
            }
        }
        return model;
    }

    /**
     * creates the WorkspaceModel.
     * @param file file
     * @return IBaseModel instance of the concrete, loaded model
     */
    private IBaseModel createWorkspaceModel(IFile file)
    {
        AbstractModelMpeFw model = ConfigurationFactory.getConfiguration().instantiateModel(file);
        InputStream stream = null;
        try
        {
            stream = file.getContents();
            model.load(stream, false);
        }
        catch (CoreException e)
        {
            ConfigurationFactory.getConfiguration().logException(e);
        }
        try
        {
            stream.close();
        }
        catch (IOException e)
        {
            ConfigurationFactory.getConfiguration().logException(e);
        }
        return model;
    }

    /**
     *
     * @see org.eclipse.pde.internal.ui.editor.context.InputContext#dispose()
     */
    public void dispose()
    {
        AbstractModelMpeFw model = (AbstractModelMpeFw) getModel();
        if (model != null)
        {
            model.dispose();
        }
        super.dispose();
    }

    /**
     *
     * @see org.eclipse.pde.internal.ui.editor.context.InputContext#flushModel(org.eclipse.jface.text.IDocument)
     */
    protected void flushModel(IDocument doc)
    {
        // if model is dirty, flush its content into
        // the document so that the source editor will
        // pick up the changes.
        if (!(getModel() instanceof IEditable)) return;
        IEditable editableModel = (IEditable) getModel();
        if (editableModel.isEditable() == false) return;
        if (editableModel.isDirty() == false) return;
        try
        {
            StringWriter swriter = new StringWriter();
            PrintWriter writer = new PrintWriter(swriter);
            editableModel.save(writer);
            writer.flush();
            swriter.close();
            doc.set(swriter.toString());
        }
        catch (IOException e)
        {
            ConfigurationFactory.getConfiguration().logException(e);
        }
    }

    /**
     *
     * @see org.eclipse.pde.internal.ui.editor.context.InputContext#synchronizeModel(org.eclipse.jface.text.IDocument)
     */
    protected boolean synchronizeModel(IDocument doc)
    {
        AbstractModelMpeFw model = (AbstractModelMpeFw) getModel();
        boolean cleanModel = true;
        String text = doc.get();
        try
        {
            InputStream stream = new ByteArrayInputStream(text.getBytes("UTF8")); //$NON-NLS-1$
            try
            {
                model.reload(stream, false);
            }
            catch (CoreException e)
            {
                cleanModel = false;
            }
            try
            {
                stream.close();
            }
            catch (IOException e)
            {
            }
        }
        catch (UnsupportedEncodingException e)
        {
            ConfigurationFactory.getConfiguration().logException(e);
        }
        return cleanModel;
    }

    /**
     *
     * @see org.eclipse.pde.internal.ui.neweditor.InputContext#getId()
     */
    public String getId()
    {
        return ConfigurationFactory.getConfiguration().getContextId();
    }

    /**
     * @see org.eclipse.pde.internal.ui.neweditor.context.InputContext#addTextEditOperation(java.util.ArrayList,
     *      org.eclipse.pde.core.IModelChangedEvent)
     */
    protected void addTextEditOperation(ArrayList ops, IModelChangedEvent event)
    {
    }

    /**
     * @see org.eclipse.pde.internal.ui.neweditor.context.XMLInputContext#reorderInsertEdits(java.util.ArrayList)
     */
    protected void reorderInsertEdits(ArrayList ops)
    {
    }

    /**
     * @see org.eclipse.pde.internal.ui.editor.context.InputContext#getPartitionName()
     */
    protected String getPartitionName()
    {
        return "___site_partition"; //$NON-NLS-1$
    }
}
