/*
 * aloba ag , Copyright (c) 2006
 * created 31.03.2006 by Hansjoerg
 */
package ch.aloba.mpefw.model.base;


import ch.aloba.mpefw.configuration.ConfigurationFactory;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.PlatformObject;
import org.eclipse.core.runtime.Status;
import org.eclipse.pde.core.IEditable;
import org.eclipse.pde.core.IModelChangedEvent;
import org.eclipse.pde.core.IModelChangedListener;
import org.eclipse.pde.core.ModelChangedEvent;
import org.eclipse.pde.internal.core.XMLDefaultHandler;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 * The main model object. Manges the state of the whole model.
 * @author Hansjoerg
 * @version $Revision: 1.1 $
 */
public abstract class AbstractModelMpeFw extends PlatformObject implements IModelMpeFw, IEditable
{
    private transient List m_listeners;
    private boolean m_loaded;
    protected boolean m_disposed;
    private long m_timeStamp;
    protected transient IModelObject m_rootObject;
    private transient IModelFactory m_factory;
    private boolean m_enabled = true;
    private boolean m_dirty;
    private IFile m_file;
    private boolean m_editable = true;



    public AbstractModelMpeFw(IFile file)
    {
        super();
        m_listeners = Collections.synchronizedList(new ArrayList());
        m_file = file;
    }

    public void addModelChangedListener(IModelChangedListener listener)
    {
        m_listeners.add(listener);
    }

    public void dispose()
    {
        m_disposed = true;
    }

    public void fireModelChanged(IModelChangedEvent event)
    {
        setDirty(event.getChangeType()!=IModelChangedEvent.WORLD_CHANGED);
        IModelChangedListener[] list = (IModelChangedListener[]) m_listeners
                        .toArray(new IModelChangedListener[m_listeners.size()]);
        for (int i = 0; i < list.length; i++)
        {
            IModelChangedListener listener = list[i];
            listener.modelChanged(event);
        }
    }

    public void fireModelObjectChanged(Object object, String property, Object oldValue,
                    Object newValue)
    {
        fireModelChanged(new ModelChangedEvent(this, object, property, oldValue, newValue));
    }

    public IResource getUnderlyingResource()
    {
        return null;
    }

    protected boolean isInSync(File localFile)
    {
        return localFile.exists() && localFile.lastModified() == getTimeStamp();
    }

    public final long getTimeStamp()
    {
        return m_timeStamp;
    }

    protected void updateTimeStamp(File localFile)
    {
        if (localFile.exists()) this.m_timeStamp = localFile.lastModified();
    }

    public boolean isDisposed()
    {
        return m_disposed;
    }

    public boolean isLoaded()
    {
        return m_loaded;
    }

    public boolean isDirty()
    {
        return m_dirty;
    }

    public void setDirty(boolean dirty)
    {
        m_dirty = dirty;
    }

    public void setLoaded(boolean loaded)
    {
        this.m_loaded = loaded;
    }

    public void save(PrintWriter writer)
    {
        if (isLoaded())
        {
            writer.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>"); //$NON-NLS-1$
            m_rootObject.write("", writer); //$NON-NLS-1$
        }
        setDirty(false);
    }

    public void load()
    {
        if (m_file.exists())
        {
            InputStream stream = null;
            try
            {
                stream = m_file.getContents(true);
                load(stream, false);
            }
            catch (CoreException e)
            {
            }
            finally
            {
                try
                {
                    if (stream != null) stream.close();
                }
                catch (IOException e)
                {
                }
            }
        }
        else
        {
            m_rootObject = null;
            getRootObject();
            setLoaded(true);
        }
    }

    public void removeModelChangedListener(IModelChangedListener listener)
    {
        m_listeners.remove(listener);
    }

    public void throwParseErrorsException(Throwable e) throws CoreException
    {
        Status status = new Status(IStatus.ERROR, getPluginId(), IStatus.OK, "parse Error", e);
        throw new CoreException(status);
    }

    protected SAXParser getSaxParser() throws ParserConfigurationException, SAXException,
                    FactoryConfigurationError
    {
        return SAXParserFactory.newInstance().newSAXParser();
    }

    /*
     * (non-Javadoc)
     * @see org.eclipse.pde.core.IModel#isReconcilingModel()
     */
    public boolean isReconcilingModel()
    {
        return false;
    }

    public IModelObject getRootObject()
    {
        if (m_rootObject == null)
        {
            m_rootObject = createRoot();
            m_rootObject.setModel(this);
        }
        return m_rootObject;
    }

    protected abstract IModelObject createRoot();

    public IModelFactory getFactory()
    {
        if (m_factory == null)
        {
            m_factory = createModelFactory();
        }
        return m_factory;
    }

    protected abstract IModelFactory createModelFactory();

    public String getInstallLocation()
    {
        return null;
    }

    public boolean isEditable()
    {
        return true;
    }

    public boolean isEnabled()
    {
        return m_enabled;
    }

    public boolean isValid()
    {
        if (!isLoaded() || m_rootObject == null)
        {
            return false;
        }
        return m_rootObject.isValid();
    }

    public void load(InputStream stream, boolean outOfSync) throws CoreException
    {
        try
        {
            SAXParser parser = getSaxParser();
            XMLDefaultHandler handler = new XMLDefaultHandler();
            parser.parse(stream, handler);
            processDocument(handler.getDocument());
            setLoaded(true);
            if (!outOfSync) updateTimeStamp();
        }
        catch (Exception e)
        {
            logException(e);
        }
    }

    protected void logException(Exception e) throws CoreException
    {
        ConfigurationFactory.getConfiguration().logException(e);
    }

    private void processDocument(Document doc)
    {
        Node rootNode = doc.getDocumentElement();
        if (m_rootObject == null)
        {
            // creates the root object
            getRootObject();
        }
        else
        {
            m_rootObject.reset();
        }
        m_rootObject.parse(rootNode);
    }

    public void reload(InputStream stream, boolean outOfSync) throws CoreException
    {
        if (m_rootObject != null)
        {
            m_rootObject.reset();
        }

        load(stream, outOfSync);
        fireModelChanged(new ModelChangedEvent(this, IModelChangedEvent.WORLD_CHANGED,
                        new Object[]{m_rootObject}, null));
    }

    public void setEnabled(boolean enabled)
    {
        this.m_enabled = enabled;
    }

    public boolean isInSync() {
        return isInSync(m_file.getLocation().toFile());
    }

    protected void updateTimeStamp() {
        updateTimeStamp(m_file.getLocation().toFile());
    }

    /**
     * @see ch.aloba.mpefw.model.base.IModelMpeFw#getPluginId()
     */
    public String getPluginId()
    {
        return ConfigurationFactory.getConfiguration().getPluginId();
    }

    /**
     * @see org.eclipse.pde.core.IModel#getResourceString(java.lang.String)
     */
    public String getResourceString(String key)
    {
        return ConfigurationFactory.getConfiguration().getResourceString(key);
    }
}
