/*
 * aloba ag , Copyright (c) 2006
 * created 28.03.2006 by Hansjoerg
 */
package ch.aloba.mpefw.model.base;

import java.io.PrintWriter;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.pde.core.ModelChangedEvent;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * Baseclass of model classes.
 *
 * @author Hansjoerg
 * @version $Revision: 1.1 $
 */
public abstract class AbstractModelObject implements IModelObject
{
    protected static final String STD_INDENT = "  ";

    transient IModelMpeFw m_model;
    transient IModelObject m_parent;
    protected String m_label;
    boolean m_inTheModel;

    public void setInTheModel(boolean value)
    {
        m_inTheModel = value;
    }

    public boolean isInTheModel()
    {
        return m_inTheModel;
    }

    protected void ensureModelEditable() throws CoreException
    {
        if (!m_model.isEditable()) {
            throwCoreException(""); //$NON-NLS-1$
        }
    }

    protected void firePropertyChanged(String property, Object oldValue, Object newValue)
    {
        firePropertyChanged(this, property, oldValue, newValue);
    }

    protected void firePropertyChanged(IModelObject object, String property, Object oldValue,
                    Object newValue)
    {
        if (m_model.isEditable())
        {
            m_model.fireModelObjectChanged(object, property, oldValue, newValue);
        }
    }

    protected void fireStructureChanged(IModelObject child, int changeType)
    {
        fireStructureChanged(new IModelObject[]{child}, changeType);
    }

    protected void fireStructureChanged(IModelObject[] children, int changeType)
    {
        IModelMpeFw model = getModel();
        if (model.isEditable())
        {
            model.fireModelChanged(new ModelChangedEvent(model, changeType, children, null));
        }
    }

    public String getLabel() {
        return m_label;
    }

    public IModelMpeFw getModel() {
        return m_model;
    }

    protected String getNodeAttribute(Node node, String name)
    {
        NamedNodeMap atts = node.getAttributes();
        Node attribute = null;
        if (atts != null) attribute = atts.getNamedItem(name);
        if (attribute != null) return attribute.getNodeValue();
        return null;
    }

    int getIntegerAttribute(Node node, String name)
    {
        String value = getNodeAttribute(node, name);
        if (value != null)
        {
            try
            {
                return Integer.parseInt(value);
            }
            catch (NumberFormatException e)
            {
            }
        }
        return 0;
    }

    boolean getBooleanAttribute(Node node, String name)
    {
        String value = getNodeAttribute(node, name);
        if (value != null)
        {
            return value.equalsIgnoreCase("true"); //$NON-NLS-1$
        }
        return false;
    }

    protected String getNormalizedText(String source)
    {
        String result = source.replace('\t', ' ');
        result = result.trim();

        return result;
    }

    public void parse(Node node)
    {
        m_label = getNodeAttribute(node, "label"); //$NON-NLS-1$
    }

    public void reset()
    {
        m_label = null;
    }

    public void setLabel(String newLabel) throws CoreException
    {
        ensureModelEditable();
        Object oldValue = m_label;
        m_label = newLabel;
        firePropertyChanged(P_LABEL, oldValue, newLabel);
    }

    protected void throwCoreException(String message) throws CoreException
    {
        Status status = new Status(IStatus.ERROR, m_model.getPluginId(), IStatus.OK, message, null);
        CoreException ce = new CoreException(status);
        ce.fillInStackTrace();
        throw ce;
    }

    public static String getWritableString(String source)
    {
        if (source == null) return ""; //$NON-NLS-1$
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < source.length(); i++)
        {
            char c = source.charAt(i);
            switch (c)
            {
                case '&':
                    buf.append("&amp;"); //$NON-NLS-1$
                    break;
                case '<':
                    buf.append("&lt;"); //$NON-NLS-1$
                    break;
                case '>':
                    buf.append("&gt;"); //$NON-NLS-1$
                    break;
                case '\'':
                    buf.append("&apos;"); //$NON-NLS-1$
                    break;
                case '\"':
                    buf.append("&quot;"); //$NON-NLS-1$
                    break;
                default:
                    buf.append(c);
                    break;
            }
        }
        return buf.toString();
    }

    public void restoreProperty(String name, Object oldValue, Object newValue) throws CoreException
    {
        if (name.equals(P_LABEL))
        {
            setLabel(newValue != null ? newValue.toString() : null);
        }
    }

    public void write(String indent, PrintWriter writer)
    {
    }

    public void setModel(IModelMpeFw model)
    {
        m_model = model;
    }

    public void setParent(IModelObject parent)
    {
        m_parent = parent;
    }

    public IModelObject getParent()
    {
        return m_parent;
    }
}


